import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_rect_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMRectDensityCal extends URMRectMeasureCal {
  URMRectDensityCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;
      Size urmResultSize = const Size(0, 0);

      // 添加入参

      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;
      List<UrmPoint> srcDPoints = [
        UrmPoint(x: p1.x, y: p1.y),
        UrmPoint(x: p2.x, y: p2.y)
      ];

      URMMeasureProcessResult? result =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMDen,
        rOIType: URMROIType.URMRect,
        srcDPoints: srcDPoints,
      );
      if (result != null) {
        // TODO 将返回的视图百分比区域的点集转换为全图百分比点集，然会然后绘制
        final feature = ref.feature!;

        /// TODO 只转数据类型还不够，需要将 视图区域百分比坐标转为全图百分比坐标

        ref.meta.outputs.first.unit = VidUsUnit.None;
        feature.updateFloatValue(
          ref.meta.outputs.first,
          result.resultData,
          ref.meta.outputs.first.unit,
        );
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
