import 'dart:ui';

import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';

import 'package:fis_measure/process/primitives/urm_measure/urm_rect_measure.dart';
// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';

import '../calculator.dart';

class URMRectMeasureCal extends Calculator<URMRectMeasure, double> {
  URMRectMeasureCal(
    URMRectMeasure ref,
  ) : super(ref);
  URMSimpleMeasureParams? param;
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    if (ref.feature == null) return;
    Size urmResultSize = const Size(0, 0);
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;
      //左上顶点
      final leftTopPercent =
          DPoint(p1.x < p2.x ? p1.x : p2.x, p1.y < p2.y ? p1.y : p2.y);
      //右下顶点
      final rightBottomPercent =
          DPoint(p1.x > p2.x ? p1.x : p2.x, p1.y > p2.y ? p1.y : p2.y);

      String description = "URM\n Calculating...";
      updateStringValue(description);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  /// 处理获取到的URM参数
  void handleURMSimpleMeasureParams() {
    if (ref.feature == null) return;
    Size urmResultSize = const Size(0, 0);
    try {
      if (ref.application is! URMApplication) {
        return;
      }
      final URMApplication urmApplication = ref.application as URMApplication;
      urmResultSize = Size(urmApplication.resultWidth.toDouble(),
          urmApplication.resultHeight.toDouble());
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;

      param = URMSimpleMeasureParams(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.URMRect,
        srcDPoints: [UrmPoint(x: p1.x, y: p1.y), UrmPoint(x: p2.x, y: p2.y)],
      );
    } catch (e) {
      logger.e('URM handleURMSimpleMeasureParams error: $e');
    }
  }
}
