import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_trace_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMTraceVelMeasureCal extends URMTraceMeasureCal {
  URMTraceVelMeasureCal(super.ref);

  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;

      // 添加入参
      List<UrmPoint> srcDPoints = [];
      for (var point in ref.feature!.innerPoints) {
        srcDPoints.add(UrmPoint(x: point.x, y: point.y));
      }

      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMVelMeasure,
        rOIType: URMROIType.URMTrace,
        srcDPoints: srcDPoints,
      );
      if (outresult == null) {
        return;
      }
      URMVelMeasureResult? result = outresult.velMeasureResult;

      if (result != null) {
        // TODO 将返回的视图百分比区域的点集转换为全图百分比点集，然会然后绘制
        final feature = ref.feature!;

        /// TODO 只转数据类型还不够，需要将 视图区域百分比坐标转为全图百分比坐标

        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.maxVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.minVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.meanVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.varianceVel, output.unit);
          } else if (output.name == MeasureTerms.Area) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.roiArea, output.unit);
          }
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
